[//]: <> (This readme is in the markdown format. Please preview in a markdown parser.)

# Final Fantasy MSX2: English Translation
Source code: [https://github.com/romh-acking/final-fantasy-msx2-en](https://github.com/romh-acking/final-fantasy-msx2-en)

## About
This is an English translation patch for Final Fantasy on the MSX2. If you're like most people, you've probably never heard of this port. The MSX was mostly popular in Japan and also certain parts of Europe. For the rest of the world, the MSX is a niche console for gaming, but it's not obscure like other Japanese computers. There's lively communities, documentation, etc. at your disposable. It's not NES tier, but it's decent enough. That being said, this translation received way more attention than what I was expecting for a Japanese computer translation. I appreciate all the input and help and I appreciate everyone's patience.

This game is a very faithful port of Final Fantasy for the NES / Famicom because a lot of data assets are ported 1:1. It features enhanced music (if you have the music add-on) and graphics that take advantage of the MSX2 hardware. The original game lacks any sort of source code as Nasir, the programmer, practically programmed the game in a hex editor, so Microcabin, this port's developer, had to program this port from scratch without reference. Therefore, the MSX2 port lacks most if not all of the bugs that plagued the Famicom version. Essentially, if you want to play a faithful but enhanced version of the original game, this is the way to go!

There is a catch however: if you have the attention span of post-internet user, you're probably going to hate the vanilla version of this port. Being a floppy disk game, there's loading times! The game's performance is terrible too. You know how people complain about games running less than 30 FPS? This game, I'm not exaggerating, has a frame cap of 6 FPS. You read that right. The act of walking around is simply sluggish and NPCs move twice as fast as you. It's Hoshi wo Miru Hito levels of excruciating. This port isn't the work of a Nasir Gebelli to say the least. Also, because no source code was provided, the Blackbelt/Monk class doesn't operate correctly in this version, so it's weaker compared to other ports.

There are a few options at your disposable to improve the experience:
1. [There's a patch that greatly improves the performance of the game.](https://github.com/abekermsx/Final-Fantasy-MSX-Performance-Patch). It doubles the framerate from 6 FPS to 12 FPS.
2. Emulators such as a BlueMSX have a throttle slider so you can increase how fast the game runs.
3. Dsk2Rom converts the game to a rom which eliminates wait time. However, the game will lock up at inns where you can rest and save.

If you want an enjoyable experience, I highly recommend the first option. When going down this route, if you apply the speed patch after applying the translation patch, while the game may boot fine, you'll encounter issues later in the game. This is because the translation patch expands files, so file locations are shifted around and applying the performance patch over the translation will overwrite data it shouldn't. Therefore, **it's recommended to apply the speed patch first *then* the translation patch**.

## Patching Instructions
Included in this archive is a [Beat](https://www.romhacking.net/utilities/893/) patch. If you get a checksum error while patching, you might want to try this patcher and its override functionality:
* [https://www.romhacking.net/patch/](https://www.romhacking.net/patch/)
* [https://www.marcrobledo.com/RomPatcher.js/](https://www.marcrobledo.com/RomPatcher.js/)

## Running the Game on an Emulator
For all emulators, you'll need a disk 1 and a disk 2. Disk 1 will contain the game data, while disk 2 (the user disk) contains save data. A lot of sites offer a download for this disk, but it's technically not necessary as there's a process in-game to create the disk. During the game's prologue, press the "Select" key (End key) to go through the process. You will need to mount a file that you wish to have the save data occupy.

During the game, you will be prompted to switch disks. If the game prompts for the user disk, do so and press the Spacebar. And likewise for the game disk.

### BlueMSX
Open BlueMSX and load the first disk into drive A. Switch the disk when prompted.

### OpenMSX
Follow this YouTube tutorial:

[https://www.youtube.com/watch?v=6MvhuTiLJqc](https://www.youtube.com/watch?v=6MvhuTiLJqc)

### RetroArch
Inside the directory where you have your disks, create an M3U file where each line is occupied with the paths of the disks. When prompted to switch sides, go through the RetroArch menus to set the disk number.

## Translation Notes
This translation uses the NES localization's script. It's a great translation for the time but okay by today's standards. People suggested using ChaosRush's translation which is far more accurate. I chose the NES localization as it's tailored to fit within the port's space restrictions. Being a floppy disk, everything is stored in RAM which made working on this game difficult. Additionally, the way the game handles pointers conflates the problem. 

The NES script wasn't completely carried over 1:1 though:
* The Japanese spell names are more optimized for space in this port so Japanese spells with the same name map to one text entry. I suppose I could've adjusted how spell names are mapped to match the NES version though.
* I didn't censor the graphics because that just seemed silly, so churches are still churches. To accommodate for this, church related text is from the prototype build.
* Inns in the NES script have text about holding reset after saving, which is a NES hardware specific precaution. To accommodate for this, the prototype script entry was used instead.

Hate my choices for the script? I don't mind. To be honest, I'm not exactly proud of my work here, and using the NES script was more of an easy workaround. Fortunately, the translation is open source so you're free to edit the script as you see fit. Just keep in mind, you must abide by the repo's license.

Hacking the game was quite a struggle. While I really shouldn't have done this, I did most my debugging in BlueMSX of all emulators. I operated solely on execution breakpoints, memory dumps, and static analysis. Amusingly, Emulicious, an excellent Z80 emulator, added support for the MSX2 after the translation was completed. I could've also used MAME or OpenMSX too. Oh well. 

It's a minor miracle that the project was completely at all. Every step of the way I didn't think I had the tools at my disposal. Here's some of the major milestones: 
* **Text dumping/inserting:** Apparently, the script encoding is the same as the Famicom version. Same exact pointer values too, amusingly enough. Reverse engineering the Famicom release and then diffing the MSX release proved to be a great help. While the dialogue had a pointer table, the rest of the text had hardcoded pointers everywhere. Fortunately, all text have a shared text printing function so once that was found, locating the pointers was easy. I was using BlueMSX at the time so I found it solely with execution breakpoints.
* **Graphic decompression:** I reverse engineered the game's absolutely backwards and inefficient compression. The text decompression code is so slow that stopping the debugger will surely land you in the middle of the decompression logic. The output is an obscure format, which I only found viewable in the tool Tile Molester.
* **File extraction**: The game uses a custom file system for anti-piracy purposes. There was a post by Tellah detailing the format on romhacking.net. Sylvester from msx.org was also helpful. Apparently, the game hardcodes the battle graphics sector load as an additional anti-piracy measure. 
* **Intro**: The intro text is stored as graphics for some reason. The code was terribly optimized so I was able to optimize the code and insert my own logic to expand the amount of lines to display.
* **DTE Compression:** Being a floppy disk game, there's only a few options for code caves. While a file has slack space, the slack space almost always isn't in RAM. I resorted to optimizing code and leveraging the no longer needed Japanese diacritic function to insert my asm hacks. I also noticed the disk image's boot sector was always loaded into memory, which I used to store the DTE table.

All in all, it's understandable why there wasn't a proper translation for this game until now. It doesn't help that there's already a translated NES version.

## Manual
You can check out the original scans here:

[https://archive.org/details/FinalFantasy1989JP/](https://archive.org/details/FinalFantasy1989JP/)

At the time of this writing, there's a manual scan online, but the last few pages are missing. The table of contents reference a magic list, but it isn't included in the scan. Additionally, the staple alignment in the scan also corroborates there's pages missing in the scan.

Because the manual's terminology is based on the localization, it is not accurate. Therefore, I've included the raw translation on GitHub. The repo also includes higher quality versions of the translated manual:

[https://github.com/romh-acking/final-fantasy-msx2-en-manual](https://github.com/romh-acking/final-fantasy-msx2-en-manual)

Also, the last page's text below MicroCabin's logo was too blurry to make out, therefore, an address for MicroCabin found online was chosen as a replacement.

In the last handful of pages, there's a list of items with text superimposed on their art. With some careful wording and editing, I was able to seamlessly cover most the previous Japanese text, but not all. If you know where the original art work is from, let me know.

## Changelog
* 2023 March 25th: 1.0
    * Initial release

## Credits

### Main Team
* FCandChill
    * ASM hacking
    * Utilities
    * Manual
* Her-saki
    * Manual translation
* TheMajinZenki
    * Manual translation

### Support
* Quibus#2625
* TFH|Fony#1064 

### Beta Testers
* Morhamms357#3536
* BigFive#4043
* Sylver#7257
* ArjanBakker#8999
* Creepy#3091
* PeachPie#4994
* xboxown#7869
